@@ -1,6 +1,4 @@
-from python_linq import From
-from python_linq.core import Grouping
-from python_linq.linq_exceptions import *
+from python_linq import From, Grouping, NoSuchElementError
 import unittest
 
 class TestBasicFunctions(unittest.TestCase):
@@ -60,7 +58,7 @@ class TestBasicFunctions(unittest.TestCase):
         )
 
         self.assertEqual(
-            From(subject).average(lambda x: x*x),
+            From(subject).select(lambda x: x*x).average(),
             11
         )
 
@@ -70,7 +68,7 @@ class TestBasicFunctions(unittest.TestCase):
             { "value": 3 }
         ]
         self.assertEqual(
-            From(subject).average(lambda x: x["value"]),
+            From(subject).select(lambda x: x["value"]).average(),
             2
         )
 
@@ -467,6 +465,22 @@ class TestBasicFunctions(unittest.TestCase):
             { "value": 2 }
         )
 
+    def test_argmax(self):
+        subject = [1, 2, 3, 4]
+        self.assertEqual(
+            From(subject).argmax(lambda x: x),
+            4
+        )
+
+        subject = [
+            { "value" : 1 },
+            { "value" : 2 }
+        ]
+        self.assertEqual(
+            From(subject).argmax(lambda x: x["value"]),
+            { "value": 2 }
+        )
+
     def test_max(self):
         subject = [1, 2, 3, 4]
         self.assertEqual(
@@ -479,10 +493,26 @@ class TestBasicFunctions(unittest.TestCase):
             { "value" : 2 }
         ]
         self.assertEqual(
-            From(subject).max(lambda x: x["value"]),
+            From(subject).select(lambda x: x["value"]).max(),
             2
         )
 
+    def test_argmin(self):
+        subject = [1, 2, 3, 4]
+        self.assertEqual(
+            From(subject).argmin(lambda x: x),
+            1
+        )
+
+        subject = [
+            { "value" : 1 },
+            { "value" : 2 }
+        ]
+        self.assertEqual(
+            From(subject).argmin(lambda x: x["value"]),
+            { "value": 1 }
+        )
+
     def test_min(self):
         subject = [1, 2, 3, 4]
         self.assertEqual(
@@ -495,7 +525,7 @@ class TestBasicFunctions(unittest.TestCase):
             { "value" : 2 }
         ]
         self.assertEqual(
-            From(subject).min(lambda x: x["value"]),
+            From(subject).select(lambda x: x["value"]).min(),
             1
         )
 
@@ -559,7 +589,7 @@ class TestBasicFunctions(unittest.TestCase):
         )
 
         self.assertEqual(
-            From(subject).sum(lambda x: x*x),
+            From(subject).select(lambda x: x*x).sum(),
             55
         )
 
@@ -569,7 +599,7 @@ class TestBasicFunctions(unittest.TestCase):
             { "value": 3 }
         ]
         self.assertEqual(
-            From(subject).sum(lambda x: x["value"]),
+            From(subject).select(lambda x: x["value"]).sum(),
             6
         )
 
@@ -651,7 +681,8 @@ class TestBasicFunctions(unittest.TestCase):
                 outerKey = lambda x: x["userid"],
                 innerTransform = lambda x: x["name"],
                 outerTransform = lambda x: x["grade"]
-            ).select(lambda x: x.inner).toList(),
+            )
+            .select(lambda x: x.inner).toList(),
             
             ['Jakob', 'Johan']
         )
@@ -675,6 +706,8 @@ class TestBasicFunctions(unittest.TestCase):
         result = From(subjA).groupJoin(
             subjB,
             lambda x: x,
+            lambda x: x,
+            lambda x: x,
             lambda x: x
         ).select(lambda x: x.outer).toList()
         expected = [[], [2, 2], [3]]
@@ -862,7 +895,7 @@ class TestBasicFunctions(unittest.TestCase):
         )
 
         self.assertListEqual(
-            From([1, 2, 3]).union([3, 4, 5], transform = lambda x: x + 1).toList(),
+            From([1, 2, 3]).union([3, 4, 5]).select(lambda x: x+1).toList(),
             [2, 3, 4, 5, 6]
         )
 
@@ -875,5 +908,43 @@ class TestBasicFunctions(unittest.TestCase):
 
         self.assertTrue(True)
 
-if __name__ == '__main__':
-    unittest.main()
\ No newline at end of file
+    def test_skip(self):
+        self.assertListEqual(
+            From([1,2,3,4,5,6,7]).skip(3).toList(),
+            [4,5,6,7]
+        )
+
+    def test_skipWhile(self):
+        self.assertListEqual(
+            From([1,2,3,4,5,6,7]).skipWhile(lambda x: x < 3).toList(),
+            [3,4,5,6,7]
+        )
+
+        self.assertListEqual(
+            From([1,2,3,4,5,6,7]).skipWhile(lambda x: x % 3 != 0).toList(),
+            [3,4,5,6,7]
+        )
+
+    def test_toDict(self):
+        self.assertDictEqual(
+            From([1,2,3,4]).toDict(lambda x: str(x*x)),
+            {
+                '1': 1,
+                '4': 2,
+                '9': 3,
+                '16': 4
+            }
+        )
+
+        self.assertDictEqual(
+            From([1,2,3,4]).toDict(lambda x: str(x), transform = lambda x: x*x),
+            {
+                '1': 1,
+                '2': 4,
+                '3': 9,
+                '4': 16
+            }
+        )
+
+        with self.assertRaises(KeyError):
+            From([1,2,3,4]).toDict(lambda x: str(x % 3))
\ No newline at end of file
